# coding: utf-8
"""

    OurTkinter.py

    Copyright (c) 2016-2018, Masatsuyo Takahashi, KEK-PF

"""
import sys
import os

import tkinter as Tk
from tkinter import TkVersion
# from tkinter.simpledialog import Dialog
from tkinter import filedialog as FileDialog
from tkinter.scrolledtext import ScrolledText
import tkinter.font as Font
import tkinter.ttk as ttk
# import tkinter.tix as tix

from TkDndWrapper       import TkDND
from TkSupplements      import tk_set_icon_portable     # for backward compatibility

if sys.version_info > (3,6):
    if sys.version_info > (3,7,1) or sys.version_info < (3,7) and sys.version_info > (3,6,7):
        # this case includes Python 3.7.1 or 3.6.7 or later
        from idlelib.tooltip    import TooltipBase
        class ToolTip(TooltipBase):
            def __init__(self, button, text):
                TooltipBase.__init__(self, button)
                self.text = text
    else:
        # this case includes Python 3.7.0 or 3.6.6 or earlier
        from idlelib.tooltip    import ToolTip

else:
    from idlelib.ToolTip    import ToolTip
# from TkSupplements      import tk_set_icon_portable

def is_empty_var( var ):
    return var is None or var == '' or var[0] == '<' or var == 'None' 

def checkFolder( parent, folder_var, folder_entry ):
    # TODO: include these checks in a custom widget class
    def set_error( entry ):
        entry.config( fg='red' )
        entry.focus_force()

    ok_ = False
    folder = folder_var.get()
    if is_empty_var( folder ):
        set_error( folder_entry )
        MessageBox.showerror( "Input Folder Error", "The Input Folder is required.", parent=parent )
    elif not os.path.exists( folder ):
        set_error( folder_entry )
        MessageBox.showerror( "Input Folder Error", "'%s' does not exist." % in_folder, parent=parent )
    elif not os.path.isdir( folder ):
        set_error( folder_entry )
        MessageBox.showerror( "Input Folder Error", "'%s' is not a folder." % in_folder, parent=parent )
    else:
        folder_entry.config( fg='black' )
        ok_ = True

    return ok_


class Dialog(Tk.Toplevel):

    '''Class to open dialogs.

    This class is intended as a base class for custom dialogs
    '''

    def __init__(self, parent, title=None, auto_geometry=True, geometry_cb=None, visible=True, block=True ):

        '''Initialize a dialog.

        Arguments:

            parent -- a parent window (the application window)

            title -- the dialog title
        '''
        Tk.Toplevel.__init__(self, parent)

        self.withdraw() # remain invisible for now
        # If the master is not viewable, don't
        # make the child transient, or else it
        # would be opened withdrawn
        if parent.winfo_viewable():
            self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        body = Tk.Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        if auto_geometry:
            self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                      parent.winfo_rooty()+50))

        if geometry_cb is not None:
            geometry_cb()

        if not visible:
            return

        self._show(block=block)

    def _show(self, block=True):
        self.deiconify() # become visible now

        self.initial_focus.focus_set()

        # wait for window to appear on screen before calling grab_set
        self.wait_visibility()
        self.grab_set()

        if not block:
            self.parent.after( 500, self.ok )

        self.wait_window(self)

    def destroy(self):
        '''Destroy the window'''
        self.initial_focus = None
        Tk.Toplevel.destroy(self)

    #
    # construction hooks

    def body(self, master):
        '''create dialog body.

        return widget that should have initial focus.
        This method should be overridden, and is called
        by the __init__ method.
        '''
        pass

    def buttonbox( self, frame=None ):
        '''add standard button box.

        override if you do not want the standard buttons
        '''

        if frame is None:
            box = Tk.Frame(self)
            box.pack()
        else:
            box = frame

        w = Tk.Button(box, text="OK", width=10, command=self.ok, default=Tk.ACTIVE)
        w.pack(side=Tk.LEFT, padx=5, pady=5)
        w = Tk.Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=Tk.LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        try:
            self.apply()
        finally:
            self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        if self.parent is not None:
            self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):
        '''validate the data

        This method is called automatically to validate the data before the
        dialog is destroyed. By default, it always validates OK.
        '''

        return 1 # override

    def apply(self):
        '''process the data

        This method is called automatically to process the data, *after*
        the dialog is destroyed. By default, it does nothing.
        '''

        pass # override
